@@ -1,4 +1,4 @@
-/* $OpenBSD: pem_lib.c,v 1.47 2018/05/13 10:47:54 tb Exp $ */
+/* $OpenBSD: pem_lib.c,v 1.49 2019/09/06 17:41:05 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -263,6 +263,16 @@ check_pem(const char *nm, const char *name)
 	    !strcmp(name, PEM_STRING_PKCS7))
 		return 1;
 
+#ifndef OPENSSL_NO_CMS
+	if (strcmp(nm, PEM_STRING_X509) == 0 &&
+	    strcmp(name, PEM_STRING_CMS) == 0)
+		return 1;
+
+	/* Allow CMS to be read from PKCS#7 headers */
+	if (strcmp(nm, PEM_STRING_PKCS7) == 0 &&
+	    strcmp(name, PEM_STRING_CMS) == 0)
+		return 1;
+#endif
 
 	return 0;
 }
@@ -613,7 +623,8 @@ PEM_write_bio(BIO *bp, const char *name, const char *header,
 	i = j = 0;
 	while (len > 0) {
 		n = (int)((len > (PEM_BUFSIZE * 5)) ? (PEM_BUFSIZE * 5) : len);
-		EVP_EncodeUpdate(&ctx, buf, &outl, &(data[j]), n);
+		if (!EVP_EncodeUpdate(&ctx, buf, &outl, &(data[j]), n))
+			goto err;
 		if ((outl) && (BIO_write(bp, (char *)buf, outl) != outl))
 			goto err;
 		i += outl;
